#ifndef COMMON_C_INCLUDED
#define	COMMON_C_INCLUDED

#ifndef	COMMON_H_INCLUDED
#error	Inclusion of 'common.c' allowed only after 'common.h'.
#endif

const char *Jlabels[4] = {"J[1][1]=","\tJ[1][2]=","J[2][1]=","\tJ[2][2]="};
const char *Jinvlabels[4] = {"J^{-1}[1][1]=","\tJ^{-1}[1][2]=",
	"J^{-1}[2][1]=","\tJ^{-1}[2][2]="};

static vboolean
mat2Inv(matrix2_t dst,matrix2_t src)
{
	vec_t idet = src[0][0] * src[1][1] - src[1][0] * src[0][1];
	if(IsZero(idet))
		return false;
	idet = 1 / idet;
	dst[0][0] = idet * src[1][1];
	dst[0][1] = - idet * src[0][1];
	dst[1][0] = - idet * src[1][0];
	dst[1][1] = idet * src[0][0];
	return true;
}

/*
 * System to solve.
 */
__inline vec_t	f1(vec_t  x,vec_t  y) { return x * x - y; }
__inline vec_t	f2(vec_t  x,vec_t  y) { return -sin(x) + y; }
static void
getF(vec2_t dst,const vec2_t X)
{
	dst[0] = f1(X[0],X[1]);
	dst[1] = f2(X[0],X[1]);
}

/*
 * Jacobi matrix must be specified explicitly.
 */
__inline vec_t	df1dx(vec_t  x,vec_t  y) { return 2*x; }
__inline vec_t	df2dx(vec_t  x,vec_t  y) { return -cos(x); }
__inline vec_t	df1dy(vec_t  x,vec_t  y) { return -1; }
__inline vec_t	df2dy(vec_t  x,vec_t  y) { return 1; }
static void
getJ(matrix2_t dst,const vec2_t X)
{
	dst[0][0] = df1dx(X[0],X[1]);
	dst[0][1] = df1dy(X[0],X[1]);
	dst[1][0] = df2dx(X[0],X[1]);
	dst[1][1] = df2dy(X[0],X[1]);
}

/*
 * First approximation.
 */
#define	FIRST_APPROX	{1, 1}

#endif	/* !COMMON_C_INCLUDED */
