#ifndef	COMMON_H_INCLUDED
#define	COMMON_H_INCLUDED

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* =======================================================================
 * Some types and macros from LibV framework library.
 */

#define	FLOAT_ZERO_EPSILON				((vec_t)(1e-10))
#define	FLOAT_ZERO_EPSILON_SQUARE		((vec_t)(1e-20))

#define	IsZeroEpsilon(v,eps) \
	(fabs(v) <= eps ? true : false)

#define	IsZero(v) \
	IsZeroEpsilon(v, FLOAT_ZERO_EPSILON)

typedef	double	vec_t;
typedef	vec_t	vec2_t[2];
typedef vec_t	matrix2_t[2][2];
#ifndef	__cplusplus
typedef	int	vboolean;
#define	false	0
#define	true	(!false)
#else
#define	vboolean bool
#endif	/* __cplusplus */

#define	vec2Set(vec,x,y) \
{ \
	(vec)[0] = (x); \
	(vec)[1] = (y); \
}

#define	vec2Zero(vec) \
	vec2Set(vec,0,0)

#define	vec2Copy(dst,src) \
{ \
	(dst)[0] = (src)[0]; \
	(dst)[1] = (src)[1]; \
}

#define	vec2Add(dst,v1,v2) \
{ \
	(dst)[0] = (v1)[0] + (v2)[0]; \
	(dst)[1] = (v1)[1] + (v2)[1]; \
}

#define	vec2Sub(dst,v1,v2) \
{ \
	(dst)[0] = (v1)[0] - (v2)[0]; \
	(dst)[1] = (v1)[1] - (v2)[1]; \
}

#define	vec2Scale(dst,vec,scale) \
{ \
	(dst)[0] = (vec)[0] * (scale); \
	(dst)[1] = (vec)[1] * (scale); \
}

#define	vec2Dot(v1,v2)	((v1)[0] * (v2)[0] + (v1)[1] * (v2)[1])

#define	vec2Abs(vec)	(vec_t)(sqrt(vec2Dot(vec,vec)))

#define	mat2Unit(mat) \
{ \
	(mat)[1][0] = (mat)[0][1] = 0.0; \
	(mat)[0][0] = (mat)[1][1] = 1.0; \
}

#define	mat2Add(dst, m1, m2) \
{ \
	(dst)[0][0] = (m1)[0][0] + (m2)[0][0]; \
	(dst)[0][1] = (m1)[0][1] + (m2)[0][1]; \
	(dst)[1][0] = (m1)[1][0] + (m2)[1][0]; \
	(dst)[1][1] = (m1)[1][1] + (m2)[1][1]; \
}

#define	mat2Scale(m,scale) \
{ \
	(m)[0][0] *= (scale); \
	(m)[0][1] *= (scale); \
	(m)[1][0] *= (scale); \
	(m)[1][1] *= (scale); \
}

#define	mat2VecMul(dst,mat,v) \
{ \
	(dst)[0] = (mat)[0][0] * (v)[0] + (mat)[0][1] * (v)[1]; \
	(dst)[1] = (mat)[1][0] * (v)[0] + (mat)[1][1] * (v)[1]; \
}

#define	mat2MatMul(dst,m1,m2) \
{ \
	(dst)[0][0] = (m1)[0][0] * (m2)[0][0] + (m1)[0][1] * (m2)[1][0]; \
	(dst)[0][1] = (m1)[0][0] * (m2)[0][1] + (m1)[0][1] * (m2)[1][1]; \
	(dst)[1][0] = (m1)[1][0] * (m2)[0][0] + (m1)[1][1] * (m2)[1][0]; \
	(dst)[1][1] = (m1)[1][0] * (m2)[0][1] + (m1)[1][1] * (m2)[1][1]; \
}

/* =======================================================================
 * Problem-specific stuff.
 */

/*
 * Maximum allowed number of iterations.
 */
#define	MAX_ITERATIONS			1e+4

/*
 * This distance indicates divergence of iteration process.
 */
#define	MAX_DISTANCE			1e+60
#define	MAX_DISTANCE_SQUARED	(MAX_DISTANCE * MAX_DISTANCE)

/*
 * Accuracy to reach during solution process.
 */
#define	DESIRED_ACCURACY		1e-6
#define	DESIRED_ACCURACY_SQUARED (DESIRED_ACCURACY * DESIRED_ACCURACY)

#define vec2Dump(v,label1,label2,laststr) \
	printf("%s%g%s%g%s",(label1),(v)[0],(label2),(v)[1],(laststr));

__inline vec_t
isNear(vec2_t v1, vec2_t v2, vec_t accuracy_squared)
{
	vec2_t v1v2;
	vec2Sub(v1v2, v1, v2);
	return (vec2Dot(v1v2, v1v2) <= accuracy_squared) ? true : false;
}

__inline vec_t
isAway(vec2_t v1, vec2_t v2)
{
	vec2_t v1v2;
	vec2Sub(v1v2, v1, v2);
	return (vec2Dot(v1v2, v1v2) > MAX_DISTANCE_SQUARED) ? true : false;
}

__inline void
mat2Dump(matrix2_t m,const char **labels)
{
	if(labels)
	{
		vec2Dump(m[0],labels[0],labels[1],"\n");
		vec2Dump(m[1],labels[2],labels[3],"\n");
	}
	else
	{
		vec2Dump(m[0],"","\t","\n");
		vec2Dump(m[1],"","\t","\n");
	}
}

#endif /* !COMMON_H_INCLUDED */
